@@ -5,7 +5,7 @@
 """
 
 from typing import (
-    Union, Iterable, Sequence, Tuple, List, Dict,
+    Union, Iterable, Sequence, Tuple, List, Dict, Any,
     Iterator, Optional, IO, overload, cast as typing_cast
 )
 from pathlib import Path
@@ -15,6 +15,8 @@ import logging
 import collections
 import itertools
 
+from progress.bar import Bar as ProgressBar
+
 from delphin import util
 from delphin import tsdb
 from delphin import interface
@@ -53,6 +55,10 @@ class FieldMapper(object):
     """
     A class for mapping between response objects and test suites.
 
+    If *source* is given, it is the test suite providing the inputs
+    used to create the responses, and it is used to provide some
+    contextual information that may not be present in the response.
+
     This class provides two methods for mapping responses to fields:
 
     * :meth:`map` -- takes a response and returns a list of (table,
@@ -83,7 +89,7 @@ class FieldMapper(object):
         affected_tables: list of tables that are affected by the
             processing
     """
-    def __init__(self):
+    def __init__(self, source: tsdb.Database = None):
         # the parse keys exclude some that are handled specially
         self._parse_keys = '''
             ninputs ntokens readings first total tcpu tgc treal words
@@ -102,7 +108,7 @@ class FieldMapper(object):
             user host os start end items status
         '''.split()
         self._parse_id = -1
-        self._runs = {}
+        self._runs: Dict[int, Dict[str, Any]] = {}
         self._last_run_id = -1
 
         self.affected_tables = '''
@@ -110,6 +116,15 @@ class FieldMapper(object):
             update fold score
         '''.split()
 
+        self._i_id_map: Dict[int, int] = {}
+        if source:
+            pairs = typing_cast(List[Tuple[int, int]],
+                                source.select_from(
+                                    'parse',
+                                    ('parse-id', 'i-id'),
+                                    cast=True))
+            self._i_id_map.update(pairs)
+
     def map(self, response: interface.Response) -> Transaction:
         """
         Process *response* and return a list of (table, rowdata) tuples.
@@ -145,8 +160,16 @@ class FieldMapper(object):
     def _map_parse(self, response: interface.Response) -> tsdb.ColumnMap:
         patch: tsdb.ColumnMap = {}
         # custom remapping, cleanup, and filling in holes
-        patch['i-id'] = response.get('keys', {}).get('i-id', -1)
-        self._parse_id = max(self._parse_id + 1, patch['i-id'])
+        keys = response.get('keys', {})
+        if 'i-id' in keys:
+            patch['i-id'] = keys['i-id']
+        elif 'parse-id' in keys and keys['parse-id'] in self._i_id_map:
+            patch['i-id'] = self._i_id_map[keys['parse-id']]
+        else:
+            patch['i-id'] = -1
+        i_id = patch['i-id']
+        assert isinstance(i_id, int)  # for type-checker's benefit, mainly
+        self._parse_id = max(self._parse_id + 1, i_id)
         patch['parse-id'] = self._parse_id
         patch['run-id'] = response.get('run', {}).get('run-id', -1)
         if 'tokens' in response:
@@ -605,7 +628,7 @@ class Table(tsdb.Relation):
             Row(10)
             >>> next(table.select('i-id', 'i-input'))
             Row(10, 'It rained.')
-            >>> next(table.select('i-id', 'i-input'), cast=False)
+            >>> next(table.select('i-id', 'i-input', cast=False))
             ('10', 'It rained.')
         """
         indices = tuple(map(self._field_index.__getitem__, names))
@@ -809,9 +832,9 @@ class TestSuite(tsdb.Database):
             selector: a pair of (table_name, column_name) that specify
                 the table and column used for processor input (e.g.,
                 `('item', 'i-input')`)
-            source (:class:`TestSuite`, :class:`Table`): test suite or
-                table from which inputs are taken; if `None`, use the
-                current test suite
+            source (:class:`~delphin.tsdb.Database`): test suite from
+                which inputs are taken; if `None`, use the current
+                test suite
             fieldmapper (:class:`FieldMapper`): object for
                 mapping response fields to [incr tsdb()] fields; if
                 `None`, use a default mapper for the standard schema
@@ -836,7 +859,7 @@ class TestSuite(tsdb.Database):
         if source is None:
             source = self
         if fieldmapper is None:
-            fieldmapper = FieldMapper()
+            fieldmapper = FieldMapper(source=source)
         index = tsdb.make_field_index(source.schema[input_table])
 
         affected = set(fieldmapper.affected_tables).intersection(self.schema)
@@ -845,21 +868,35 @@ class TestSuite(tsdb.Database):
 
         key_names = [f.name for f in source.schema[input_table] if f.is_key]
 
+        bar = None
+        if not logger.isEnabledFor(logging.INFO):
+            with tsdb.open(source.path, input_table) as fh:
+                total = sum(1 for _ in fh)
+            if total > 0:
+                bar = ProgressBar('Processing', max=total)
+
         for row in source[input_table]:
             datum = row[index[input_column]]
             keys = [row[index[name]] for name in key_names]
             keys_dict = dict(zip(key_names, keys))
             response = cpu.process_item(datum, keys=keys_dict)
+
             logger.info(
                 'Processed item {:>16}  {:>8} results'
                 .format(tsdb.join(keys), len(response['results']))
             )
+            if bar:
+                bar.next()
+
             for tablename, data in fieldmapper.map(response):
                 _add_row(self, tablename, data, buffer_size)
 
         for tablename, data in fieldmapper.cleanup():
             _add_row(self, tablename, data, buffer_size)
 
+        if bar:
+            bar.finish()
+
         tsdb.write_database(self, self.path, gzip=gzip)
 
 
